@@ -1,9 +1,9 @@
 <?php
-// Change_ldappass 1.5 (May 15, 2003)
+// Change_ldappass 1.6 (July 26, 2003)
 //  
-// Copyright Simon Annetts 2000 - 2002. simon@ateb.co.uk 
+// Copyright Simon Annetts 2000 - 2003. simon@ateb.co.uk 
 // PLEASE SEND QUESTIONS COMMENTS AND CHANGES TO MAINTAINOR.
-// New Maintainor: Raymond Ferguson foo@share-foo.com
+// Maintainor: Raymond Ferguson foo@share-foo.com
 // 
 // This software may be used, modified and distributed freely
 // providing this copyright notice remains intact at the head 
@@ -20,6 +20,7 @@
 //
 // Portions of code, ideas, suggestions and bug fixes have been posted by:
 //
+// Simon Annetts (Original Author)
 // Matt Butcher
 // Henrique de Moraes Holschuh
 // David Hall
@@ -28,8 +29,10 @@
 // Todd Lyons
 // Greg Schueler 
 // Beat
-// Georg Lehner (internationalization)
+// Georg Lehner 
 // Ray Ferguson
+// Maciej Skalski
+// Marcio Merlone 
 // 
 // Apologies if I've missed anyone :-)
 
@@ -228,6 +231,12 @@ function change_ldappass_go($debug) {
 				if ($debug) array_push($Messages, _("Password type is {SHA}"));
 				$cpass=ldap_sha_passwd($cp_oldpass);
 				break;
+			case "{smd5":
+				if ($debug) array_push($Messages, _("Password type is {SMD5}"));
+				$hash = base64_decode($lpass) ;
+				$salt = substr($hash, 16);
+				$cpass = base64_encode(pack("H*", md5($cp_oldpass . $salt)).$salt);
+				break;
       default:                        // Use plain text password
         $cpass=$cp_oldpass;
         $lpass=$storedpass;     // Override $lpass as it is truncated from the original
@@ -250,6 +259,10 @@ function change_ldappass_go($debug) {
 			case "{sha":
 				$newpass="{SHA}".ldap_sha_passwd($cp_newpass);
 				break;			
+			case "{smd5":
+				$newpass="{SMD5}".ldap_smd5_passwd($cp_newpass);
+				break;
+				
 			// more password types should go here ;-) and the functions to drive them below.
 	    default:                        // Use plain text password
       	$newpass=$cp_newpass;
@@ -307,6 +320,12 @@ function ldap_md5_passwd($clear_pw) {
 	return base64_encode(pack("H*", md5($clear_pw)));	
 }
 
+function ldap_smd5_passwd($clear_pw) {
+	 $salt = myhash_keyge_s2k($clear_pw, 4);
+	 $new_password = base64_encode(pack("H*", md5($clear_pw . $salt)) . $salt);
+	 return $new_password ;
+}
+
 // Generate salted crypt passwords
 function ldap_crypt_passwd($password,$salt,$stype) {
 	if ($stype=="MD5") 	return crypt($password,substr($salt,0,12)); 						//MD5 uses 12 chr salt
@@ -326,6 +345,14 @@ function ldap_generate_salt($stype) {
 	if ($stype=="DES")	return $salt;
 }
 
+function myhash_keyge_s2k($pass, $bytes ){
+	// This and md5 implimentation of the Salted S2K algorithm as 
+	// specified in the OpenPGP document (RFC 2440).
+	// basically a non mhash dependant version of mhash_keygen_s2k
+	$salt=substr(pack("h*", md5(mt_rand())), 0, 8);
+	return substr(pack("H*", md5($salt . $pass)), 0, $bytes);
+}
+
 function ldap_debug_print_array($array) {
 	$out="<br>--------------------------------------------------------<br>\n";
   $out.=ldap_debug_print_array1($array,"");
